"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareSummary = exports.compareCandidates = exports.limitTasks = exports.semaphore = exports.refineCallstack = exports.getCallstackDiff = exports.refineFingerprint = exports.getUrlForEndpoint = void 0;
function getUrlForEndpoint(url, cfg, endpoint) {
    const queryPorts = cfg.endpoints[endpoint];
    let queryPort;
    let baseProtocol = url.startsWith('https') ? 'https' : 'http';
    queryPort = queryPorts.find((portObj) => {
        return portObj.protocol === baseProtocol;
    });
    if (!queryPort) {
        throw new Error('cannot find query url');
    }
    return `${url}:${queryPort.port}/${endpoint}`;
}
exports.getUrlForEndpoint = getUrlForEndpoint;
function refineFingerprint(raw, fingerprint) {
    return {
        fingerprint,
        first_seen: parseInt(raw[fingerprint]['head(fingerprint;first_seen)'][0]),
        callstack: refineCallstack(raw[fingerprint]['head(callstack)']),
        classifiers: raw[fingerprint]['head(classifiers)'],
        dates: raw[fingerprint]['range(timestamp)'],
        count: raw[fingerprint].count,
    };
}
exports.refineFingerprint = refineFingerprint;
function getCallstackDiff(cstack, ostack) {
    const mainCallstackDiff = [];
    const candidateCallstackDiff = [];
    // process original callstack compared to candidate
    ostack.forEach((frame) => {
        // If the frame doesn't exist in the candidate, mark it as a "deleted" frame.
        if (cstack.indexOf(frame) < 0) {
            mainCallstackDiff.push([frame, 'deleted']);
        }
        else {
            mainCallstackDiff.push([frame, 'nothing']);
        }
    });
    // process candidate callstack compared to original
    cstack.forEach((frame, index) => {
        const frameLocation = ostack.indexOf(frame);
        // If the frame doesn't exist in the original, mark it as a "added" frame.
        if (frameLocation < 0) {
            candidateCallstackDiff.push([frame, 'added']);
        }
        else if (frameLocation !== index) {
            candidateCallstackDiff.push([frame, 'moved']);
        }
        else {
            candidateCallstackDiff.push([frame, 'nothing']);
        }
    });
    return { mainCallstackDiff, candidateCallstackDiff };
}
exports.getCallstackDiff = getCallstackDiff;
function refineCallstack(cstack) {
    if (cstack.length === 0)
        return [];
    return JSON.parse(cstack[0]).frame;
}
exports.refineCallstack = refineCallstack;
function semaphore(limit, ms) {
    const capacity = limit;
    let currTasks = 0;
    return {
        hasCapacity: () => {
            return currTasks < capacity;
        },
        addTask: () => {
            currTasks++;
        },
        removeTask: () => {
            currTasks--;
        },
        ms: ms,
    };
}
exports.semaphore = semaphore;
function timeout(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
async function limitTasks(taskFns, localLimit, globalLimit, requestState) {
    // Loop to process only a few tasks at a time.
    while (taskFns.length >= localLimit) {
        if (requestState.cancelRequest) {
            break;
        }
        if (!globalLimit.hasCapacity()) {
            await timeout(globalLimit.ms);
            continue;
        }
        const promiseFns = taskFns.splice(0, localLimit + 1);
        const promises = promiseFns.map((fn) => fn());
        await Promise.all(promises);
    }
    if (requestState.cancelRequest) {
        return Promise.resolve();
    }
    // Finish off any remaining tasks.
    const promises = taskFns.map((fn) => fn());
    return Promise.all(promises);
}
exports.limitTasks = limitTasks;
/**
 * For sorting by distance (smallest distance first)
 * and then by number of instances (more instances first)
 * */
function compareCandidates(a, b) {
    if (a.distance == b.distance) {
        return b.count - a.count;
    }
    return a.distance - b.distance;
}
exports.compareCandidates = compareCandidates;
/**
 * For sorting by number of candidate instances (more instances first)
 * */
function compareSummary(a, b) {
    return b.candidateInstances - a.candidateInstances;
}
exports.compareSummary = compareSummary;
//# sourceMappingURL=utilities.js.map